clear all;
close all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Programming in Matlab %%%%%%%
%% by Javier Hernandez Rivera %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Outline
% - Introduction
% - Managing Files
% - Variables:
%     . Matrices
%     . Strings 
%     . Cell arrays
%     . Structures
% - Control Statements
% - Logical Operators
% - I/O
% - Drawing
% - Efficiency
     

%% Introduction
% Where is it?
%     http://ist.mit.edu/services/software
%     \\hub.media.mit.edu\winhub\APPS\ML_Software\Matlab\2010A_Windows
% Software: MATtrix LABoratory
%     . Command window, workspace (whos, path)
%     . help, doc
%     . unix commands: cd, ls, rm, pwd...


%% Managing files
% Scripts
%     clear all, close all
% Functions
%        function [arg1 arg2] = nameFunc(param1, param2)
%             global param3
%             ...
%             f1 = inline('x+3','x');
%        end
 

%% Variables
% Case sensitive, underscores, and digits and numbers
%  e.g. abc_d12 ~= ABC_d12


%%  - Matrices
% Scalar
a = 1;

% Vector
b = [1; 2; 3];

% Matrix
A = [1 2 3; 4 5 6; 7 8 9];

%Initializations
Id = eye(3)
C = diag(A)
B = diag(diag(A))
uniform = rand(1,3)
normal = randn(1,3)
magic(3)

% Access to matrices
%     . A(row, col)
A(1,1)
%     . colon notation: A(1:3,2)
A(1:3,2)


% Operators
% +, -, *, ^, ', /, \ (A\b solves Ax = b)
 
A'
A + B
A * B
A .* B


%  Functions
%     sin, cos, tan, exp, log, abs, sqrt, sign, round, floor, ceil, fact, abs
%     max, min, sort, sum, prod, median, mean, std, find, linspace
%     eig, inv, det, size, norm, repmat, cov
%   Everything exists!	

cos(pi)
log(0)
sign(-1)
round(0.6)
floor(0.6)
ceil(0.6)

a = [1:4 4:-1:1]
sort(a)
median(a)
[p1 p2] = find(a == 4)
linspace(2,5,10)

[U V] = eig(A)

%% Strings
string1 = 'abc'
string1(2) = []


%% Cell arrays
list_strings = {'a1', 'a2', 'a3'}
list_strings{1}
strcmp(list_strings{1},list_strings{2})



%% Structures
STR(1).int = 1
STR(1).string = 'tmp'



%% Control Statements
% IF(my_func(x, y) > pi)
%     ...
% ELSEIF(x < y)
%     ...
% ELSE
%     ...
% END

% FOR i=1:10
%    ...
% END

% WHILE i=1:10
%    ...
% END

% SWITCH x
%     CASE 1 
%         ...
%     OTHERWISE
% END


%% LOGICAL OPERATORS
% ==, ~=, <=, >=, &, |, ~, any, all
4 == 5
any([0 0 0 1])
all([0 0 0 1])

%% I/O
% Output
%   disp, fprintf
fprintf('Hello world %d!\n', 1);
disp(A);

% Input
%   input
%   fscanf, fgets
%   load, fopen, csvread

load('points')


%% DRAWING
% figure
% plot3, scatter3, hist, meshgrid, mesh
% title, xlabel, ylabel, text, axis, hold on

figure;
scatter(X(:,1),X(:,2),'filled');
hold on;
plot(X(:,1),X(:,2),'xr');
xlabel('X');
ylabel('Y');
title('Data');

%% EFFICIENCY	
% Preallocate and vectorize

z = 0;
for i = [1:5]
    for j = [1:5]
        bad(i,j) = rand;
    end
end


z = 0;
medium = zeros(5,5);
for i = [1:5]
    for j = [1:5]
        medium(i,j) = rand;
    end
end


good = rand(5,5);

       
%% Optimization
% tic; ... t1 = toc;
% profile on; profile viewer; profile off; (time spent in each function)
%  
% keyboard, pause, dbcont, dbup, dbdown (control of execution)
% mex files (c or c++ code)